/**
 * classe Concessionaire
 * @author Matthias Colin
 * @version 1.0 (9/06/2009)
 */

import java.awt.Color;
import java.util.*;

public class Concessionnaire {
	private Map<String, List<Voiture>> voituresDisponibles;

	/**
	 * constructeur de la classe Concessionaire
	 */
	public Concessionnaire() {
		voituresDisponibles = new HashMap<String, List<Voiture>>(); 
	}

	/**
	 * ajout d'une voiture dans la concession
	 * @param voiture la voiture à ajouter
	 */
	public void ajouterVoiture(Voiture voiture) {
		String clef;
		clef = cle(voiture);
		if (voituresDisponibles.containsKey(clef)) {
			voituresDisponibles.get(clef).add(voiture);
		} else {
			List<Voiture> nouvelleListe;
			nouvelleListe = new LinkedList<Voiture>();
			nouvelleListe.add(voiture);
			voituresDisponibles.put(clef, nouvelleListe);
		}
	}

	/** 
	 * recherche de voiture disponible dans la concession à partir
	 * d'une marque et un modèle
	 * @param marque la marque à rechercher
	 * @param modele le modèle à rechercher
	 * @return la liste des voitures trouvées
	 */	
	public List<Voiture> rechercherVoitureMarqueModele(String marque, String
			modele) {
		List<Voiture> liste;
		liste = voituresDisponibles.get(cle(marque,modele));
		if (liste == null) {
			liste = new LinkedList<Voiture>();
		} 
		return liste;
	}

	/** 
	 * recherche de voiture disponible dans la concession à partir
	 * d'une marque d'un modèle et d'une couleur
	 * @param marque la marque à rechercher
	 * @param modele le modèle à rechercher
	 * @param couleur la couleur à rechercher
	 * @return la liste des voitures trouvées
	 */	
	public List<Voiture> rechercherVoitureMarqueModeleCouleur(String marque, 
			String modele, Color couleur) {
		List<Voiture> liste;
		liste = new LinkedList<Voiture>();
		for (Voiture voiture : rechercherVoitureMarqueModele(marque, modele)) {
			if (voiture.getCouleur().equals(couleur)) {
				liste.add(voiture);
			}
		}
		return liste;
	}

	/** 
	 * recherche de voiture disponible dans la concession à partir
	 * d'une couleur
	 * @param couleur la couleur à rechercher
	 * @return la liste des voitures trouvées
	 */	
	public List<Voiture> rechercherVoitureCouleur(Color couleur) {
		List<Voiture> liste;
		liste = new LinkedList<Voiture>();
	
		// parcours des voitures disponibles pour chaque marque et modèle
		for (List<Voiture> listeUneMarqueUnModele : voituresDisponibles.values()) {
			for (Voiture voiture : listeUneMarqueUnModele) {
				if (voiture.getCouleur().equals(couleur)) {
					liste.add(voiture);
				}
			}
		}
		return liste;
	}

	/**
	 * @return clé d'indexation à partir d'une voiture
	 */
	private String cle(Voiture voiture) {
		return cle(voiture.getMarque(), voiture.getModele());
	}

	/**
	 * @return clé d'indexation à partir d'une marque et un modèle
	 */
	private String cle(String marque, String modele) {
		return (marque + "/" + modele);
	}
}
